/**
 * @file libleaftag/source.h Source object and functions
 *
 * @Copyright (C) 2005-2006 Christian Hammond
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef _LEAFTAG_SOURCE_H_
#define _LEAFTAG_SOURCE_H_

#include <libleaftag/object.h>

typedef struct _LtSource      LtSource;
typedef struct _LtSourcePriv  LtSourcePriv;
typedef struct _LtSourceClass LtSourceClass;

struct _LtSource
{
	LtObject parent_object;
	LtSourcePriv *priv;
};

struct _LtSourceClass
{
	LtObjectClass parent_class;

	/* Signals */
	void (*tags_changed)(LtSource *source);

	void (*reserved_1)(void);
	void (*reserved_2)(void);
	void (*reserved_3)(void);
	void (*reserved_4)(void);
};

#define LT_TYPE_SOURCE (lt_source_get_type())

/** Cast @c obj to a LtSource. */
#define LT_SOURCE(obj) \
		(G_TYPE_CHECK_INSTANCE_CAST((obj), LT_TYPE_SOURCE, LtSource))

/** Cast @c klass to a LtSource. */
#define LT_SOURCE_CLASS(klass) \
		(G_TYPE_CHECK_CLASS_CAST((klass), LT_TYPE_SOURCE, LtSourceClass))

/** Check if @c obj is a LtSource. */
#define LT_IS_SOURCE(obj) \
		(G_TYPE_CHECK_INSTANCE_TYPE((obj), LT_TYPE_SOURCE))

/** Check if @c klass is a LtSourceClass. */
#define LT_IS_SOURCE_CLASS(klass) \
		(G_TYPE_CHECK_CLASS_TYPE((klass), LT_TYPE_SOURCE))

/** Get the class for the LtSource @c obj. */
#define LT_SOURCE_GET_CLASS(obj) \
		(G_TYPE_INSTANCE_GET_CLASS((obj), LT_TYPE_SOURCE, LtSourceClass))

/**
 * Returns the GType for LtSource.
 *
 * @return LtSource's GType.
 */
GType lt_source_get_type(void);

LtSource *lt_create_source(const char *uri);
LtSource *lt_source_lookup(const char *uri);
GList *lt_source_lookup_many(GList *uris);

/**
 * Deletes a source and any associations made to tags.
 *
 * @param source The source to delete.
 */
void lt_source_delete(LtSource *source);

void lt_source_set_uri(LtSource *source, const char *new_uri);
const char *lt_source_get_uri(const LtSource *source);

/**
 * Returns the filename of a source, if the source has a "file" schema.
 *
 * If the source does not have a "file" schema, this will display a
 * warning and return NULL.
 *
 * @param source The source.
 *
 * @return The filename, or NULL if the source does not have a file schema.
 */
const char *lt_source_get_filename(const LtSource *source);

/**
 * Retrieves the schema of the source's URI.
 *
 * @param source The source.
 *
 * @return The schema of the source's URI.
 */
const char *lt_source_get_schema(const LtSource *source);

/**
 * Tags a source with one or more tag objects.
 *
 * @param source The source to tag.
 * @param tags   The list of tag objects to tag the source with.
 */
void lt_source_tag(LtSource *source, GList *tags);

/**
 * Tags a source with one or more tag names.
 *
 * @param source The source to tag.
 * @param tags   The list of tag names to tag the source with.
 */
void lt_source_tag_with_names(LtSource *source, GList *tag_names);

/**
 * Removes one or more tags from a source.
 *
 * @param source The source to remove tags from.
 * @param tags   The list of tag objects to remove from the source.
 */
void lt_source_untag(LtSource *source, GList *tag);

/**
 * Removes one or more tags by name from a source.
 *
 * @param source    The source to remove tags from.
 * @param tag_names The list of tag names to remove from the source.
 */
void lt_source_untag_with_names(LtSource *source, GList *tag_names);

/**
 * Returns a list of tags on the specified source.
 *
 * @param source The source.
 *
 * @return The tag objects associated with the source.
 */
GList *lt_source_get_tags(const LtSource *source);

/**
 * Returns a list of all tagged sources.
 *
 * @return A list of all tagged sources.
 */
GList *lt_get_all_sources(void);

/**
 * Returns a list of tagged sources with the specified schema.
 *
 * @param schema The schema.
 *
 * @return The list of tagged sources.
 */
GList *lt_get_sources_with_schema(const char *schema);

/**
 * Returns a list of sources tagged with the specified tags.
 *
 * This optionally takes a schema to match against. It can be NULL.
 *
 * @param tags   The list of tag objects to match against.
 * @param schema The optional schema to match against, or NULL.
 *
 * @return The list of sources.
 */
GList *lt_get_sources_with_tags(GList *tags, const char *schema);

/**
 * Returns a list of sources tagged with the specified tag names.
 *
 * This optionally takes a schema to match against. It can be NULL.
 *
 * @param tag_names The list of tag names to match against.
 * @param schema    The optional schema to match against, or NULL.
 *
 * @return The list of sources.
 */
GList *lt_get_sources_with_tag_names(GList *tag_names, const char *schema);

#endif /* _LEAFTAG_SOURCE_H_ */
