/**
 * @file libleaftag/tag.h Tag object and functions
 *
 * @Copyright (C) 2005-2006 Christian Hammond
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef _LEAFTAG_TAG_H_
#define _LEAFTAG_TAG_H_

#include <libleaftag/object.h>

typedef struct _LtTag      LtTag;
typedef struct _LtTagPriv  LtTagPriv;
typedef struct _LtTagClass LtTagClass;

struct _LtTag
{
	LtObject parent_object;
	LtTagPriv *priv;
};

struct _LtTagClass
{
	LtObjectClass parent_class;

	/* Signals */
	void (*sources_changed)(LtTag *tag);

	void (*reserved_1)(void);
	void (*reserved_2)(void);
	void (*reserved_3)(void);
	void (*reserved_4)(void);
};

#define LT_TYPE_TAG (lt_tag_get_type())

/** Cast @c obj to a LtTag. */
#define LT_TAG(obj) \
		(G_TYPE_CHECK_INSTANCE_CAST((obj), LT_TYPE_TAG, LtTag))

/** Cast @c klass to a LtTag. */
#define LT_TAG_CLASS(klass) \
		(G_TYPE_CHECK_CLASS_CAST((klass), LT_TYPE_TAG, LtTagClass))

/** Check if @c obj is a LtTag. */
#define LT_IS_TAG(obj) \
		(G_TYPE_CHECK_INSTANCE_TYPE((obj), LT_TYPE_TAG))

/** Check if @c klass is a LtTagClass. */
#define LT_IS_TAG_CLASS(klass) \
		(G_TYPE_CHECK_CLASS_TYPE((klass), LT_TYPE_TAG))

/** Get the class for the LtTag @c obj. */
#define LT_TAG_GET_CLASS(obj) \
		(G_TYPE_INSTANCE_GET_CLASS((obj), LT_TYPE_TAG, LtTagClass))

/**
 * Returns the GType for LtTag.
 *
 * @return LtTag's GType.
 */
GType lt_tag_get_type(void);

LtTag *lt_create_tag(const char *tag_name);
LtTag *lt_tag_lookup(const char *tag_name);
GList *lt_tag_lookup_many(GList *tag_names);

/**
 * Deletes a tag and any associations made to sources.
 *
 * @param tag The tag to delete.
 */
void lt_tag_delete(LtTag *tag);

const char *lt_tag_get_name(const LtTag *tag);

void lt_tag_set_hidden(LtTag *tag, gboolean hidden);
gboolean lt_tag_get_hidden(const LtTag *tag);

void lt_tag_set_description(LtTag *tag, const char *description);
const char *lt_tag_get_description(const LtTag *tag);

void lt_tag_set_image(LtTag *tag, const char *image_path);
const char *lt_tag_get_image(const LtTag *tag);

/**
 * Returns a list of sources tagged with the specified tag.
 *
 * This optionally takes a schema to match against. It can be NULL.
 *
 * If you want to search with multiple tags at once, use
 * lt_get_sources_with_tags()
 *
 * @param tag    The tag to match against.
 * @param schema The optional schema to match against, or NLL.
 *
 * @return The list of sources.
 */
GList *lt_tag_get_sources(const LtTag *tag, const char *schema);

/**
 * Returns a list of all known tags.
 *
 * @return A list of all known tags.
 */
GList *lt_get_all_tags(void);

/**
 * Returns a list of all visible tags.
 *
 * @return A list of all visible tags.
 */
GList *lt_get_visible_tags(void);

/**
 * Returns whether or not the specified tag name is valid.
 *
 * Valid tags must not have the following set of characters: \t\n\r()[]<>+,'"
 *
 * @param tag_name The name of the tag to check.
 *
 * @return TRUE if the tag name is valid, otherwise FALSE.
 */
gboolean lt_get_tag_name_valid(const char *tag_name);

/**
 * Returns whether or not all the tag names in a list are valid.
 *
 * Valid tags must not have the following set of characters: \t\n\r()[]<>+,'"
 *
 * @param tag_names The list of tag names to check.
 *
 * @return TRUE if the tag names are all valid, otherwise FALSE.
 */
gboolean lt_get_tag_names_valid(GList *tag_names);

#endif /* _LEAFTAG_TAG_H_ */
